package com.xy.satoken;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.xy.swagger.Knife4jConfig;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.config.annotation.InterceptorRegistration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.List;

/**
 * 单体boot鉴权
 */
@Component
@AllArgsConstructor
public class SaTokenWebMvcConfigurer extends HandlerInterceptorAdapter implements WebMvcConfigurer {

    private SaTokenAuthorizeConfig saTokenAuthorizeConfig;

    private SaTokenAuthorize saTokenAuthorize;

    @Override
    @SneakyThrows
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        String permission;
        String satoken;
        if ("/authorize/satokenAuthorize".equals(request.getRequestURI())) {
            //接口形式鉴权（如网关调用），从请求参数获取权限信息
            BufferedReader streamReader = new BufferedReader(new InputStreamReader(request.getInputStream(), "UTF-8"));
            StringBuilder sb = new StringBuilder();
            String inputStr;
            while ((inputStr = streamReader.readLine()) != null) {
                sb.append(inputStr);
            }
            JSONObject jsonObject = JSONUtil.parseObj(sb);
            permission = jsonObject.getStr("permission");
            satoken = jsonObject.getStr("satoken");
        } else {
            //其他调用，从请求地址和请求头获取权限信息
            permission = request.getRequestURI();
            satoken = request.getHeader(saTokenAuthorizeConfig.getTokenName());
        }
        return saTokenAuthorize.check(satoken, permission);
    }


    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        InterceptorRegistration interceptorRegistration = registry.addInterceptor(this)
                .excludePathPatterns(Knife4jConfig.swaggerExcludes)
                .addPathPatterns("/**");
        List<String> excludePaths = saTokenAuthorizeConfig.getExcludePaths();
        if (excludePaths != null) {
            excludePaths.forEach(excludePath -> interceptorRegistration.excludePathPatterns("/**" + excludePath));
        }
    }

}
